function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import merge from 'lodash/merge';
import { ERROR_CODE_CREATE_ANNOTATION, ERROR_CODE_CREATE_REPLY, ERROR_CODE_DELETE_ANNOTATION, ERROR_CODE_EDIT_ANNOTATION, ERROR_CODE_FETCH_ANNOTATION, ERROR_CODE_FETCH_ANNOTATIONS, ERROR_CODE_FETCH_REPLIES, PERMISSION_CAN_CREATE_ANNOTATIONS, PERMISSION_CAN_DELETE, PERMISSION_CAN_EDIT, PERMISSION_CAN_VIEW_ANNOTATIONS, PERMISSION_CAN_RESOLVE } from '../constants';
import MarkerBasedApi from './MarkerBasedAPI';
import { formatComment } from './utils';
export default class Annotations extends MarkerBasedApi {
  constructor() {
    super(...arguments);

    _defineProperty(this, "successHandler", data => {
      if (this.isDestroyed() || typeof this.successCallback !== 'function') {
        return;
      } // There is no response data when deleting an annotation


      if (!data) {
        this.successCallback();
        return;
      } // We don't have entries when updating/creating an annotation


      if (!data.entries) {
        // Check if the response is a comment (result of createAnnotationReply)
        if (data.type && data.type === 'comment') {
          this.successCallback(formatComment(data));
          return;
        }

        this.successCallback(this.formatReplies(data));
        return;
      } // Check if the response is the replies of an annotation (result of getAnnotationReplies)


      if (data.entries.length && data.entries[0].type === 'comment') {
        const replies = data.entries.map(formatComment);
        this.successCallback(_objectSpread({}, data, {
          entries: replies
        }));
        return;
      }

      const annotations = data.entries.map(this.formatReplies);
      this.successCallback(_objectSpread({}, data, {
        entries: annotations
      }));
    });
  }

  /**
   * Formats annotation replies' comment data for use in components.
   *
   * @param {Annotation} annotation - An individual annotation entry from the API
   * @return {Annotation} Updated annotation
   */
  formatReplies(annotation) {
    if (!annotation.replies || !annotation.replies.length) {
      return annotation;
    }

    return _objectSpread({}, annotation, {
      replies: annotation.replies.map(formatComment)
    });
  }
  /**
   * Formats the annotations api response to usable data
   * @param {Object} data the api response data
   */


  getUrl() {
    return "".concat(this.getBaseApiUrl(), "/undoc/annotations");
  }

  getUrlForId(annotationId) {
    return "".concat(this.getUrl(), "/").concat(annotationId);
  }

  getUrlWithRepliesForId(annotationId) {
    return "".concat(this.getUrlForId(annotationId), "/replies");
  }

  createAnnotation(fileId, fileVersionId, payload, permissions, successCallback, errorCallback) {
    this.errorCode = ERROR_CODE_CREATE_ANNOTATION;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_CREATE_ANNOTATIONS, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    const defaults = {
      description: {
        type: 'reply'
      },
      file_version: {
        id: fileVersionId,
        type: 'file_version'
      }
    };
    this.post({
      id: fileId,
      data: {
        data: merge(defaults, payload)
      },
      errorCallback,
      successCallback,
      url: this.getUrl()
    });
  }

  updateAnnotation(fileId, annotationId, permissions, payload, successCallback, errorCallback) {
    this.errorCode = ERROR_CODE_EDIT_ANNOTATION;
    const {
      message,
      status
    } = payload;

    if (message) {
      try {
        this.checkApiCallValidity(PERMISSION_CAN_EDIT, permissions, fileId);
      } catch (e) {
        errorCallback(e, this.errorCode);
        return;
      }
    }

    if (status) {
      try {
        this.checkApiCallValidity(PERMISSION_CAN_RESOLVE, permissions, fileId);
      } catch (e) {
        errorCallback(e, this.errorCode);
        return;
      }
    }

    this.put({
      id: fileId,
      data: {
        data: {
          description: message ? {
            message
          } : undefined,
          status
        }
      },
      errorCallback,
      successCallback,
      url: this.getUrlForId(annotationId)
    });
  }

  deleteAnnotation(fileId, annotationId, permissions, successCallback, errorCallback) {
    this.errorCode = ERROR_CODE_DELETE_ANNOTATION;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_DELETE, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    this.delete({
      id: fileId,
      errorCallback,
      successCallback,
      url: this.getUrlForId(annotationId)
    });
  }

  getAnnotation(fileId, annotationId, permissions, successCallback, errorCallback, shouldFetchReplies) {
    this.errorCode = ERROR_CODE_FETCH_ANNOTATION;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_VIEW_ANNOTATIONS, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    const requestData = shouldFetchReplies ? {
      params: {
        fields: 'replies'
      }
    } : undefined;
    this.get({
      id: fileId,
      errorCallback,
      successCallback,
      url: this.getUrlForId(annotationId),
      requestData
    });
  }

  getAnnotations(fileId, fileVersionId, permissions, successCallback, errorCallback, limit, shouldFetchAll, shouldFetchReplies) {
    this.errorCode = ERROR_CODE_FETCH_ANNOTATIONS;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_VIEW_ANNOTATIONS, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    const requestData = _objectSpread({
      file_id: fileId,
      file_version_id: fileVersionId
    }, shouldFetchReplies ? {
      fields: 'replies'
    } : null);

    this.markerGet({
      id: fileId,
      errorCallback,
      limit,
      requestData,
      shouldFetchAll,
      successCallback
    });
  }

  getAnnotationReplies(fileId, annotationId, permissions, successCallback, errorCallback) {
    this.errorCode = ERROR_CODE_FETCH_REPLIES;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_VIEW_ANNOTATIONS, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    this.get({
      id: fileId,
      errorCallback,
      successCallback,
      url: this.getUrlWithRepliesForId(annotationId)
    });
  }

  createAnnotationReply(fileId, annotationId, permissions, message, successCallback, errorCallback) {
    this.errorCode = ERROR_CODE_CREATE_REPLY;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_CREATE_ANNOTATIONS, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    this.post({
      id: fileId,
      data: {
        data: {
          message
        }
      },
      errorCallback,
      successCallback,
      url: "".concat(this.getUrlWithRepliesForId(annotationId), "?file_id=").concat(fileId)
    });
  }

}
//# sourceMappingURL=Annotations.js.map