function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * 
 * @file Helper for the box threadedComments API
 * @author Box
 */
import MarkerBasedApi from './MarkerBasedAPI';
import { PERMISSION_CAN_COMMENT, PERMISSION_CAN_DELETE, PERMISSION_CAN_EDIT, ERROR_CODE_CREATE_COMMENT, ERROR_CODE_UPDATE_COMMENT, ERROR_CODE_DELETE_COMMENT, ERROR_CODE_FETCH_COMMENT, ERROR_CODE_FETCH_COMMENTS, PERMISSION_CAN_RESOLVE, ERROR_CODE_FETCH_REPLIES, ERROR_CODE_CREATE_REPLY } from '../constants';
import { formatComment } from './utils';

class ThreadedComments extends MarkerBasedApi {
  constructor() {
    super(...arguments);

    _defineProperty(this, "successHandler", data => {
      if (this.isDestroyed() || typeof this.successCallback !== 'function') {
        return;
      } // There is no response data when deleting a comment


      if (!data) {
        this.successCallback();
        return;
      } // We don't have entries when updating/creating a comment


      if (!data.entries) {
        this.successCallback(formatComment(data));
        return;
      }

      const comments = data.entries.map(formatComment);
      this.successCallback(_objectSpread({}, data, {
        entries: comments
      }));
    });
  }

  /**
   * API URL for comments
   *
   * @param {string} [fileId]
   * @return {string} base url for comments
   */
  getUrl(fileId) {
    return "".concat(this.getBaseApiUrl(), "/undoc/comments").concat(fileId ? "?file_id=".concat(fileId) : '');
  }
  /**
   * API URL for specific comment
   *
   * @param {string} [commentId]
   * @return {string} base url for specific comment
   */


  getUrlForId(commentId) {
    return "".concat(this.getUrl(), "/").concat(commentId);
  }
  /**
   * API URL for specific comment
   *
   * @param {string} commentId
   * @param {string} [fileId]
   * @return {string}  base url for specific comment replies
   */


  getUrlWithRepliesForId(commentId, fileId) {
    return "".concat(this.getUrlForId(commentId), "/replies").concat(fileId ? "?file_id=".concat(fileId) : '');
  }
  /**
   * Formats the threaded comments api response to usable data
   * @param {Object} data the api response data
   */


  /**
   * API for creating a comment on a file
   *
   * @param {BoxItem} file - File object for which we are creating a comment
   * @param {string} message - Comment message
   * @param {Function} successCallback - Success callback
   * @param {Function} errorCallback - Error callback
   * @return {void}
   */
  createComment(_ref) {
    let {
      file,
      message,
      successCallback,
      errorCallback
    } = _ref;
    this.errorCode = ERROR_CODE_CREATE_COMMENT;
    const {
      id,
      permissions
    } = file;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_COMMENT, permissions, id);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    this.post({
      id,
      url: this.getUrl(id),
      data: {
        data: {
          message
        }
      },
      successCallback,
      errorCallback
    });
  }
  /**
   * API for updating a comment
   *
   * @param {string} fileId - File id for which we are updating a comment
   * @param {string} commentId - Comment to be edited
   * @param {FeedItemStatus} status - Comment status
   * @param {string} message - Comment message
   * @param {BoxCommentPermission} permissions - The known permissions of the comment we're updating
   * @param {Function} successCallback - Success callback
   * @param {Function} errorCallback - Error callback
   * @return {void}
   */


  updateComment(_ref2) {
    let {
      fileId,
      commentId,
      status,
      message,
      permissions,
      successCallback,
      errorCallback
    } = _ref2;
    this.errorCode = ERROR_CODE_UPDATE_COMMENT;

    if (message) {
      try {
        this.checkApiCallValidity(PERMISSION_CAN_EDIT, permissions, fileId);
      } catch (e) {
        errorCallback(e, this.errorCode);
        return;
      }
    }

    if (status) {
      try {
        this.checkApiCallValidity(PERMISSION_CAN_RESOLVE, permissions, fileId);
      } catch (e) {
        errorCallback(e, this.errorCode);
        return;
      }
    }

    const requestData = {
      data: {
        status,
        message
      }
    };
    this.put({
      id: fileId,
      url: this.getUrlForId(commentId),
      data: requestData,
      successCallback,
      errorCallback
    });
  }
  /**
   * API for deleting a comment or reply
   *
   * @param {string} fileId - Id of an object for which we are deleting a comment
   * @param {string} commentId - Id of the comment we are deleting
   * @param {BoxCommentPermission} permissions - The known permissions of the comment we're deleting
   * @param {Function} successCallback - Success callback
   * @param {Function} errorCallback - Error callback
   * @return {void}
   */


  deleteComment(_ref3) {
    let {
      fileId,
      commentId,
      permissions,
      successCallback,
      errorCallback
    } = _ref3;
    this.errorCode = ERROR_CODE_DELETE_COMMENT;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_DELETE, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    this.delete({
      id: fileId,
      url: this.getUrlForId(commentId),
      successCallback,
      errorCallback
    });
  }
  /**
   * API for fetching comment
   *
   * @param {string} commentId - comment id
   * @param {string} fileId - the file id
   * @param {BoxItemPermission} permissions - the permissions for the file
   * @param {Function} successCallback - the success callback
   * @param {Function} errorCallback - the error callback
   * @returns {void}
   */


  getComment(_ref4) {
    let {
      commentId,
      errorCallback,
      fileId,
      permissions,
      successCallback
    } = _ref4;
    this.errorCode = ERROR_CODE_FETCH_COMMENT;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_COMMENT, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    this.get({
      id: fileId,
      errorCallback,
      successCallback,
      url: this.getUrlForId(commentId)
    });
  }
  /**
   * API for fetching comments
   *
   * @param {string} fileId - the file id
   * @param {BoxItemPermission} permissions - the permissions for the file
   * @param {Function} successCallback - the success callback
   * @param {Function} errorCallback - the error callback
   * @param {array} fields - the fields to fetch
   * @param {string} marker the marker from the start to start fetching at
   * @param {number} limit - the number of items to fetch
   * @param {boolean} shouldFetchAll - true if should get all the pages before calling the sucessCallback
   * @param {number} repliesCount - number of replies to return, by deafult all replies all returned
   *  @returns {void}
   */


  getComments(_ref5) {
    let {
      fileId,
      permissions,
      successCallback,
      errorCallback,
      marker,
      limit,
      shouldFetchAll,
      repliesCount
    } = _ref5;
    this.errorCode = ERROR_CODE_FETCH_COMMENTS;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_COMMENT, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    this.markerGet({
      id: fileId,
      successCallback,
      errorCallback,
      marker,
      limit,
      requestData: _objectSpread({}, repliesCount ? {
        replies_count: repliesCount
      } : null),
      shouldFetchAll
    });
  }
  /**
   * @param {string} fileId - the file id
   * @param {string} commentId - id of a Comment
   * @param {BoxItemPermission} permissions - The known permissions of the comment
   * @param {Function} successCallback - the success callback
   * @param {Function} errorCallback - the error callback
   */


  getCommentReplies(_ref6) {
    let {
      fileId,
      commentId,
      permissions,
      successCallback,
      errorCallback
    } = _ref6;
    this.errorCode = ERROR_CODE_FETCH_REPLIES;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_COMMENT, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    this.get({
      id: fileId,
      errorCallback,
      successCallback,
      url: this.getUrlWithRepliesForId(commentId)
    });
  }
  /**
   * @param {string} fileId - the file id
   * @param {string} commentId - id of a Comment for which we createing Reply
   * @param {BoxItemPermission} permissions - The known permissions of the comment
   * @param {Function} successCallback - the success callback
   * @param {Function} errorCallback - the error callback
   * @param {string} message - message of the Reply
   */


  createCommentReply(_ref7) {
    let {
      fileId,
      commentId,
      permissions,
      successCallback,
      errorCallback,
      message
    } = _ref7;
    this.errorCode = ERROR_CODE_CREATE_REPLY;

    try {
      this.checkApiCallValidity(PERMISSION_CAN_COMMENT, permissions, fileId);
    } catch (e) {
      errorCallback(e, this.errorCode);
      return;
    }

    this.post({
      id: fileId,
      data: {
        data: {
          message
        }
      },
      errorCallback,
      successCallback,
      url: this.getUrlWithRepliesForId(commentId, fileId)
    });
  }

}

export default ThreadedComments;
//# sourceMappingURL=ThreadedComments.js.map