function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import * as React from 'react';
import omit from 'lodash/omit';
import classNames from 'classnames';
import MenuContext from './MenuContext';
import './Menu.scss';
/**
 * The selectors are used to identify the menu item that is selected. We need to eventually
 * rewrite this logic as there seem to be strong coupling between the selector and MenuItem
 * that we want to decouple. The span is here to allow Menu to recognize MenuItem even if it is
 * wrapped by a span coming from a tooltip.
 */

const MENU_ITEM_SELECTOR = '.menu-item:not([aria-disabled])';
const TOP_LEVEL_MENU_ITEM_SELECTOR = "ul:not(.submenu) > ".concat(MENU_ITEM_SELECTOR, ", ul:not(.submenu) > li > ").concat(MENU_ITEM_SELECTOR, ", ul:not(.submenu) > span > ").concat(MENU_ITEM_SELECTOR);
const SUBMENU_ITEM_SELECTOR = "ul.submenu > ".concat(MENU_ITEM_SELECTOR, ", ul.submenu > li > ").concat(MENU_ITEM_SELECTOR, ", ul.submenu > span > ").concat(MENU_ITEM_SELECTOR);

function stopPropagationAndPreventDefault(event) {
  event.stopPropagation();
  event.preventDefault();
}

class Menu extends React.Component {
  constructor(_props) {
    var _this;

    super(_props);
    _this = this;

    _defineProperty(this, "setInitialFocusIndex", function () {
      let props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this.props;
      const {
        initialFocusIndex,
        isHidden
      } = props;

      if (isHidden || initialFocusIndex === undefined) {
        return;
      } // If an initialFocusIndex was specified, attempt to use it to focus


      if (typeof initialFocusIndex === 'number') {
        // We do this after a timeout so that the menu is properly mounted before we attempt to focus it
        setTimeout(() => {
          _this.setFocus(initialFocusIndex);
        }, 0);
      } else if (initialFocusIndex === null) {
        // If no initial focus index is set, focus on the menu itself so that keyboard shortcut still works after a mouse click.
        setTimeout(() => {
          if (_this.menuEl) {
            _this.menuEl.focus();
          }
        }, 0);
      }
    });

    _defineProperty(this, "setMenuItemEls", () => {
      const {
        isSubmenu,
        menuItemSelector
      } = this.props;
      const selector = menuItemSelector || (isSubmenu ? SUBMENU_ITEM_SELECTOR : TOP_LEVEL_MENU_ITEM_SELECTOR); // Keep track of all the valid menu items that were rendered (querySelector since we don't want to pass ref functions to every single child)

      this.menuItemEls = this.menuEl ? [].slice.call(this.menuEl.querySelectorAll(selector)) : [];
    });

    _defineProperty(this, "getMenuItemElFromEventTarget", target => {
      let menuItemEl = null;
      let menuIndex = -1;

      for (let i = 0; i < this.menuItemEls.length; i += 1) {
        if (this.menuItemEls[i].contains(target)) {
          menuItemEl = this.menuItemEls[i];
          menuIndex = i;
          break;
        }
      }

      return {
        menuItemEl,
        menuIndex
      };
    });

    _defineProperty(this, "setFocus", index => {
      if (!this.menuItemEls.length) {
        return;
      }

      const numMenuItems = this.menuItemEls.length;

      if (index >= numMenuItems) {
        this.focusIndex = 0;
      } else if (index < 0) {
        this.focusIndex = numMenuItems - 1;
      } else {
        this.focusIndex = index;
      }

      this.menuItemEls[this.focusIndex].focus();
    });

    _defineProperty(this, "focusFirstItem", () => {
      this.setFocus(0);
    });

    _defineProperty(this, "focusLastItem", () => {
      this.setFocus(-1);
    });

    _defineProperty(this, "focusNextItem", () => {
      this.setFocus(this.focusIndex + 1);
    });

    _defineProperty(this, "focusPreviousItem", () => {
      this.setFocus(this.focusIndex - 1);
    });

    _defineProperty(this, "fireOnCloseHandler", (isKeyboardEvent, event) => {
      const {
        onClose
      } = this.props;

      if (onClose) {
        // We need to pass the event type so we know which item to focus.
        onClose(isKeyboardEvent, event);
      }
    });

    _defineProperty(this, "handleClick", event => {
      const {
        menuItemEl
      } = event.target instanceof Node ? this.getMenuItemElFromEventTarget(event.target) : {};

      if (!menuItemEl) {
        return;
      }

      this.fireOnCloseHandler(false, event);
    });

    _defineProperty(this, "handleKeyDown", event => {
      const {
        isSubmenu,
        initialFocusIndex
      } = this.props;

      switch (event.key) {
        case 'ArrowDown':
          stopPropagationAndPreventDefault(event); // If it's first keyboard event, focus on first item.

          if (initialFocusIndex === null && !this.keyboardPressed) {
            this.focusFirstItem();
          } else {
            this.focusNextItem();
          }

          break;

        case 'ArrowUp':
          stopPropagationAndPreventDefault(event);
          this.focusPreviousItem();
          break;

        case 'ArrowLeft':
          // Close submenu when arrow-left is clicked
          if (!isSubmenu) {
            return;
          }

          stopPropagationAndPreventDefault(event);
          this.fireOnCloseHandler(true, event);
          break;

        case 'Home':
        case 'PageUp':
          stopPropagationAndPreventDefault(event);
          this.focusFirstItem();
          break;

        case 'End':
        case 'PageDown':
          stopPropagationAndPreventDefault(event);
          this.focusLastItem();
          break;

        case 'Escape':
          stopPropagationAndPreventDefault(event);
          this.fireOnCloseHandler(true, event);
          break;

        case 'Tab':
          // DO NOT PREVENT DEFAULT OR STOP PROPAGATION - This should move focus natively
          this.fireOnCloseHandler(true, event);
          break;

        case ' ':
        case 'Enter':
          stopPropagationAndPreventDefault(event);

          if (event.target instanceof HTMLElement) {
            event.target.click();
          }

          break;

        default:
          break;
      }

      this.keyboardPressed = true;
    });

    this.focusIndex = 0;
    this.menuEl = null;
    this.menuItemEls = [];
  }

  componentDidMount() {
    this.setMenuItemEls();
    this.setInitialFocusIndex();
  }

  componentDidUpdate(_ref) {
    let {
      isHidden: prevIsHidden,
      children: prevChildren
    } = _ref;
    const {
      children,
      isHidden,
      isSubmenu
    } = this.props;

    if (isSubmenu && prevIsHidden && !isHidden) {
      // If updating submenu, use the current props instead of previous props.
      this.setMenuItemEls();
      this.setInitialFocusIndex(this.props);
    } // update focus index and menu item elements when the number of children changes


    if (React.Children.toArray(prevChildren).length !== React.Children.toArray(children).length) {
      const focusedMenuItemEl = this.menuItemEls[this.focusIndex];
      this.setMenuItemEls();
      const {
        menuIndex
      } = this.getMenuItemElFromEventTarget(focusedMenuItemEl);
      const isFocusedElementMissing = menuIndex === -1;
      const isFocusIndexOutOfBounds = this.focusIndex >= this.menuItemEls.length;
      this.setFocus(isFocusedElementMissing && !isFocusIndexOutOfBounds ? this.focusIndex : menuIndex);
    }
  }

  render() {
    const _this$props = this.props,
          {
      children,
      className,
      isHidden,
      setRef,
      shouldOutlineFocus
    } = _this$props,
          rest = _objectWithoutProperties(_this$props, ["children", "className", "isHidden", "setRef", "shouldOutlineFocus"]);

    const menuProps = omit(rest, ['onClose', 'initialFocusIndex', 'isSubmenu', 'menuItemSelector']);
    menuProps.className = classNames('aria-menu', className, {
      'is-hidden': isHidden,
      'should-outline-focus': shouldOutlineFocus
    });

    menuProps.ref = ref => {
      this.menuEl = ref;

      if (setRef) {
        setRef(ref);
      }
    };

    if (menuProps.role === undefined) {
      menuProps.role = 'menu';
    }

    menuProps.tabIndex = -1;
    menuProps.onClick = this.handleClick;
    menuProps.onKeyDown = this.handleKeyDown;
    return React.createElement("ul", menuProps, React.createElement(MenuContext.Provider, {
      value: {
        closeMenu: this.fireOnCloseHandler
      }
    }, children));
  }

}

_defineProperty(Menu, "defaultProps", {
  className: '',
  isSubmenu: false,
  isHidden: false
});

export default Menu;
//# sourceMappingURL=Menu.js.map