/**
 * 
 * @file Item list component
 * @author Box
 */
import React from 'react';
import classNames from 'classnames';
import { injectIntl } from 'react-intl';
import { Table, Column } from '@box/react-virtualized/dist/es/Table';
import AutoSizer from '@box/react-virtualized/dist/es/AutoSizer';
import KeyBinder from '../common/KeyBinder';
import nameCellRenderer from '../common/item/nameCellRenderer';
import iconCellRenderer from '../common/item/iconCellRenderer';
import { focus } from '../../utils/dom';
import messages from '../common/messages';
import headerCellRenderer from './headerCellRenderer';
import sizeCellRenderer from './sizeCellRenderer';
import dateCellRenderer from './dateCellRenderer';
import moreOptionsCellRenderer from './moreOptionsCellRenderer';
import { FIELD_DATE, FIELD_ID, FIELD_NAME, FIELD_SIZE, VIEW_FOLDER, VIEW_RECENTS } from '../../constants';
import '@box/react-virtualized/styles.css';
import './ItemList.scss';

const ItemList = (_ref) => {
  let {
    view,
    isSmall,
    isMedium,
    isTouch,
    rootId,
    rootElement,
    canShare,
    canDownload,
    canDelete,
    canPreview,
    canRename,
    onItemClick,
    onItemSelect,
    onItemDelete,
    onItemDownload,
    onItemRename,
    onItemShare,
    onItemPreview,
    onSortChange,
    currentCollection,
    tableRef,
    focusedRow,
    intl
  } = _ref;
  const nameCell = nameCellRenderer(rootId, view, onItemClick, onItemSelect, canPreview, isSmall, // shows details if false
  isTouch);
  const iconCell = iconCellRenderer();
  const dateCell = dateCellRenderer();
  const sizeAccessCell = sizeCellRenderer();
  const moreOptionsCell = moreOptionsCellRenderer(canPreview, canShare, canDownload, canDelete, canRename, onItemSelect, onItemDelete, onItemDownload, onItemRename, onItemShare, onItemPreview, isSmall);
  const isRecents = view === VIEW_RECENTS;
  const hasSort = view === VIEW_FOLDER;
  const {
    id,
    items = [],
    sortBy,
    sortDirection
  } = currentCollection;
  const rowCount = items.length;

  const rowClassName = (_ref2) => {
    let {
      index
    } = _ref2;

    if (index === -1) {
      return 'bce-item-header-row';
    }

    const {
      selected
    } = items[index];
    return classNames("bce-item-row bce-item-row-".concat(index), {
      'bce-item-row-selected': selected
    });
  };

  const sort = (_ref3) => {
    let {
      sortBy: by,
      sortDirection: direction
    } = _ref3;
    onSortChange(by, direction);
  };

  return React.createElement(KeyBinder, {
    id: id,
    items: items,
    columnCount: 1,
    rowCount: rowCount,
    className: "bce-item-grid",
    onRename: onItemRename,
    onShare: onItemShare,
    onDownload: onItemDownload,
    onOpen: onItemClick,
    onSelect: onItemSelect,
    onDelete: onItemDelete,
    scrollToRow: focusedRow,
    onScrollToChange: (_ref4) => {
      let {
        scrollToRow
      } = _ref4;
      return focus(rootElement, ".bce-item-row-".concat(scrollToRow));
    }
  }, (_ref5) => {
    let {
      onSectionRendered,
      scrollToRow,
      focusOnRender
    } = _ref5;
    return React.createElement(AutoSizer, null, (_ref6) => {
      let {
        width,
        height
      } = _ref6;
      return React.createElement(Table, {
        width: width,
        height: height,
        headerHeight: isSmall ? 0 : 40,
        rowHeight: 50,
        rowCount: rowCount,
        rowGetter: (_ref7) => {
          let {
            index
          } = _ref7;
          return items[index];
        },
        ref: tableRef,
        rowClassName: rowClassName,
        scrollToIndex: scrollToRow,
        sort: sort,
        sortBy: sortBy,
        sortDirection: sortDirection,
        onRowClick: (_ref8) => {
          let {
            rowData
          } = _ref8;
          return onItemSelect(rowData);
        },
        onRowsRendered: (_ref9) => {
          let {
            startIndex,
            stopIndex
          } = _ref9;
          onSectionRendered({
            rowStartIndex: startIndex,
            rowStopIndex: stopIndex
          });

          if (focusOnRender) {
            focus(rootElement, ".bce-item-row-".concat(scrollToRow));
          }
        }
      }, React.createElement(Column, {
        disableSort: true,
        dataKey: FIELD_ID,
        cellRenderer: iconCell,
        headerRole: "gridcell",
        width: isSmall ? 30 : 50,
        flexShrink: 0
      }), React.createElement(Column, {
        disableSort: !hasSort,
        label: intl.formatMessage(messages.name),
        dataKey: FIELD_NAME,
        cellRenderer: nameCell,
        headerRenderer: headerCellRenderer,
        width: 300,
        flexGrow: 1
      }), isSmall ? null : React.createElement(Column, {
        className: "bce-item-column",
        disableSort: !hasSort,
        label: isRecents ? intl.formatMessage(messages.interacted) : intl.formatMessage(messages.modified),
        dataKey: FIELD_DATE,
        cellRenderer: dateCell,
        headerRenderer: headerCellRenderer,
        width: isRecents ? 120 : 300,
        flexGrow: 1
      }), isSmall || isMedium ? null : React.createElement(Column, {
        className: "bce-item-column",
        disableSort: !hasSort,
        label: intl.formatMessage(messages.size),
        dataKey: FIELD_SIZE,
        cellRenderer: sizeAccessCell,
        headerRenderer: headerCellRenderer,
        width: 80,
        flexShrink: 0
      }), React.createElement(Column, {
        disableSort: true,
        dataKey: FIELD_ID,
        cellRenderer: moreOptionsCell,
        headerRole: "gridcell",
        width: isSmall || !canShare ? 58 : 140,
        flexShrink: 0
      }));
    });
  });
};

export default injectIntl(ItemList);
//# sourceMappingURL=ItemList.js.map