function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import PropTypes from 'prop-types';
import React, { useState } from 'react';
import { injectIntl } from 'react-intl';
import classNames from 'classnames';
import throttle from 'lodash/throttle';
import Draggable from 'react-draggable';
import IconCloud from '../../icons/general/IconCloud';
import messages from './messages';
import { getGridPosition } from './utils';

const DropShadowFilter = () => React.createElement("filter", {
  id: "drop-shadow"
}, React.createElement("feGaussianBlur", {
  in: "SourceAlpha",
  stdDeviation: "2"
}), React.createElement("feOffset", {
  dx: "2",
  dy: "2",
  result: "offsetblur"
}), React.createElement("feFlood", {
  floodColor: "black",
  floodOpacity: "0.3"
}), React.createElement("feComposite", {
  in2: "offsetblur",
  operator: "in"
}), React.createElement("feMerge", null, React.createElement("feMergeNode", null), React.createElement("feMergeNode", {
  in: "SourceGraphic"
})));

const DragCloud = (_ref) => {
  let {
    cloudSize,
    disabled,
    gameBoardSize: {
      height,
      width
    },
    gridTrackSize,
    intl: {
      formatMessage
    },
    onDrop,
    position,
    updateLiveText,
    updatePosition
  } = _ref;
  const [isMoving, setIsMoving] = useState(false);
  const dragCloudClasses = classNames('bdl-DragCloud', {
    'is-moving': isMoving
  });

  const moveLeft = () => {
    const newX = position.x - gridTrackSize;
    updatePosition(_objectSpread({}, position, {
      x: Math.max(newX, 0)
    }), true);

    if (newX < 0) {
      updateLiveText(formatMessage(messages.reachLeftEdge));
    }
  };

  const moveRight = () => {
    const maxX = width - cloudSize;
    const newX = position.x + gridTrackSize;
    updatePosition(_objectSpread({}, position, {
      x: Math.min(newX, maxX)
    }), true);

    if (newX > maxX) {
      updateLiveText(formatMessage(messages.reachRightEdge));
    }
  };

  const moveUp = () => {
    const newY = position.y - gridTrackSize;
    updatePosition(_objectSpread({}, position, {
      y: Math.max(newY, 0)
    }), true);

    if (newY < 0) {
      updateLiveText(formatMessage(messages.reachTopEdge));
    }
  };

  const moveDown = () => {
    const maxY = height - cloudSize;
    const newY = position.y + gridTrackSize;
    updatePosition(_objectSpread({}, position, {
      y: Math.min(newY, maxY)
    }), true);

    if (newY > maxY) {
      updateLiveText(formatMessage(messages.reachBottomEdge));
    }
  };

  const handleSpacebar = () => {
    const cloudStatusText = formatMessage(isMoving ? messages.cloudDropped : messages.cloudGrabbed);
    const currentPositionText = formatMessage(messages.currentPosition, getGridPosition(position, gridTrackSize));
    updateLiveText("".concat(cloudStatusText, " ").concat(currentPositionText), true);

    if (isMoving) {
      onDrop();
    }

    setIsMoving(!isMoving);
  };
  /**
   * DragCloud keyboard event handler. Supports Up/Down/Left/Right arrow keys and Spacebar
   * @param {KeyboardEvent} event - The drag event
   * @returns {void}
   */


  const onKeyDown = event => {
    if (disabled) {
      return;
    }

    if (isMoving) {
      event.preventDefault();
      event.stopPropagation();
    }

    if (event.key === ' ') {
      handleSpacebar();
    } else if (isMoving) {
      switch (event.key) {
        case 'ArrowUp':
          moveUp();
          break;

        case 'ArrowDown':
          moveDown();
          break;

        case 'ArrowLeft':
          moveLeft();
          break;

        case 'ArrowRight':
          moveRight();
          break;

        default:
          break;
      }
    }
  };
  /**
   * Reset isMoving state when DragCloud loses focus
   * @returns {void}
   */


  const onBlur = () => setIsMoving(false);
  /**
   * DragCloud drag event handler. Updates current position.
   * @param {MouseEvent} e - The drag event
   * @param {object} { x, y } - Object which contains x and y coordinate of the drag event.
   * @returns {void}
   */


  const onDrag = throttle((e, _ref2) => {
    let {
      x,
      y
    } = _ref2;
    return updatePosition({
      x,
      y
    });
  }, 100, {
    leading: true,
    trailing: true
  });
  return React.createElement(Draggable, {
    bounds: "parent",
    disabled: disabled,
    onDrag: onDrag,
    onStop: onDrop,
    position: position
  }, React.createElement("div", {
    className: dragCloudClasses,
    onBlur: onBlur,
    onKeyDown: onKeyDown,
    role: "button",
    tabIndex: 0
  }, React.createElement(IconCloud, {
    filter: {
      id: 'drop-shadow',
      definition: React.createElement(DropShadowFilter, null)
    },
    height: cloudSize,
    title: formatMessage(messages.cloudObject),
    width: cloudSize
  })));
};

DragCloud.displayName = 'DragCloud';
DragCloud.propTypes = {
  cloudSize: PropTypes.number,
  disabled: PropTypes.bool,
  gameBoardSize: PropTypes.objectOf(PropTypes.number),
  gridTrackSize: PropTypes.number,
  intl: PropTypes.any,
  position: PropTypes.objectOf(PropTypes.number).isRequired,
  onDrop: PropTypes.func,
  updateLiveText: PropTypes.func,
  updatePosition: PropTypes.func
}; // Actual export

export { DragCloud as DragCloudBase };
export default injectIntl(DragCloud);
//# sourceMappingURL=DragCloud.js.map